@@ -8,6 +8,9 @@
 #   anyone's safety, is implied in this software or documentation.
 #
 #   This specific module is licensed under the Perl Artistic License.
+#   Effective 28 January 2021, the original author and copyright holder, 
+#   Martin Hosken, has given permission to use and redistribute this module 
+#   under the MIT license.
 #
 #=======================================================================
 package PDF::Builder::Basic::PDF::Bool;
@@ -17,8 +20,8 @@ use base 'PDF::Builder::Basic::PDF::String';
 use strict;
 use warnings;
 
-our $VERSION = '3.021'; # VERSION
-my $LAST_UPDATE = '3.010'; # manually update whenever code is changed
+our $VERSION = '3.025'; # VERSION
+our $LAST_UPDATE = '3.024'; # manually update whenever code is changed
 
 =head1 NAME
 
@@ -28,7 +31,9 @@ B<true> or B<false>
 
 =head1 METHODS
 
-=head2 $b->convert($str)
+=over
+
+=item $b->convert($str)
 
 Converts a string into the string which will be stored.
 
@@ -38,7 +43,7 @@ sub convert {
     return $_[1] eq 'true';
 }
 
-=head2 $b->as_pdf()
+=item $b->as_pdf()
 
 Converts the value to a PDF output form.
 
@@ -48,4 +53,8 @@ sub as_pdf {
     return $_[0]->{'val'}? 'true': 'false';
 }
 
+=back
+
+=cut
+
 1;
