@@ -7,31 +7,33 @@
 use strict;
 use warnings;
 
-our $VERSION = '3.023'; # VERSION
-our $LAST_UPDATE = '3.016'; # manually update whenever code is changed
+our $VERSION = '3.025'; # VERSION
+our $LAST_UPDATE = '3.025'; # manually update whenever code is changed
 
 # command line flags, mutually exclusive:
 # -raw   show full output of each t-test run
 # -noOK  exclude "ok" lines so can easily spot error lines  DEFAULT
 
+ # add after filter-lzwdecode when new TIFF code finished
+ #   filter-ccittfaxdecode
 my @test_list = qw(
  00-all-usable
  01-basic
  02-xrefstm
  03-xrefstm-index
  annotate
- author-critic
- author-pod-syntax
  barcode
  bbox
  circular-references
  cmap
  content
+ content-deprecated
  cs-webcolor
  deprecations
  extgstate
  filter-ascii85decode
  filter-asciihexdecode
+ filter-lzwdecode
  filter-runlengthdecode
  font-corefont
  font-synfont
@@ -39,8 +41,10 @@ my @test_list = qw(
  font-type1
  gd
  gif
+ info
  jpg
  lite
+ named-destinations
  outline
  page
  papersizes
@@ -55,11 +59,16 @@ my @test_list = qw(
  string
  text
  tiff
+ version
  viewer-preferences
                   );
 # override full list above, and run just one or two tests
 #@test_list = qw( barcode );
 
+# moved to xt/
+#   author-critic
+#   author-pod-syntax
+
 # perl t/<name>.t will run it
 
 my $type;
